"""
Django settings for gracedb project.

Description of settings files:
    __init__.py   - this file; determines whether to use production or test
                    settings for this GraceDB instance.
    local.py      - generated by Puppet, contains paths to useful directories,
                    virtualenv, and other settings.
    secret.py     - generated by Puppet, contains secret settings like the
                    database password, API keys, etc.
    base.py       - contains the basic settings for running a GraceDB server.
                    These can be overridden in production.py or test.py. Uses
                    settings in local.py and secret.py.
    production.py - defines settings for a production GraceDB instance. Imports
                    base.py settings and overrides/adds settings.
    test.py       - defines settings for a test/development GraceDB instance.
                    Imports base.py settings and overrides/adds settings.
"""


# Import either production or test settings
try:
    from .local import IS_PRODUCTION_SERVER
    if IS_PRODUCTION_SERVER:
        settings_file = 'production'
    else:
        # If ./local.py is not found or IS_PRODUCTION_SERVER
        # is not defined, use test settings
        settings_file = 'test'
except NameError:
    settings_file = 'test'

# Addition for simdb
import socket
if (socket.gethostname() == 'simdb'):
    settings_file = 'simdb'

settings_module = __import__(settings_file, globals(), locals())

# Put these settings into the local scope
for setting in dir(settings_module):
    # Only add uppercase variables. We use lowercase as
    # temporary variables in the settings files.
    if setting == setting.upper():
        locals()[setting] = getattr(settings_module, setting)
